#!/usr/bin/python

import sys, math, cgi, os, Cookie, base64, md5
from lxml import etree

def addBearings(myLat, myLon, offset, dx):
    # walk the entries calculating bearing from lat and long
    entries = dx.xpath('entry')

    for entry in entries:
        lat = float(entry.xpath('lat/text()')[0])
        lon = float(entry.xpath('lon/text()')[0])
        bearing = calculateBearing(myLat, myLon, lat, lon)+int(offset)
        if (bearing < 0):
            bearing = bearing + 360
        if (bearing > 360):
            bearing = bearing - 360
        if (bearing <= 180):
            antiBearing = bearing + 180
        else:
            antiBearing = bearing - 180

        etree.SubElement(entry, 'bearing').text=str(bearing)
        etree.SubElement(entry, 'antibearing').text=str(antiBearing)

def calculateBearing(fromLat, fromLon, toLat, toLon):
    radians=180.0/math.pi

    fromLatr=fromLat/radians
    fromLonr=fromLon/radians

    toLatr=toLat/radians
    toLonr=toLon/radians

    dlong = toLonr - fromLonr
    dlat = toLatr - fromLatr
    arg1=math.sin(dlong) * math.cos(toLatr)
    arg2a = math.cos(fromLatr) * math.sin(toLatr)
    arg2b = math.sin(fromLatr) * math.cos(toLatr) * math.cos(dlong)
    arg2 =  arg2a -  arg2b
    bearingr = math.atan2(arg1, arg2)

    bearing = round(((360 + (bearingr * radians)) % 360))
    return bearing

def qra(myqra):
    c1 = int(myqra[0:1], 36)-10
    c2 = int(myqra[1:2], 36)-10
    c3 = int(myqra[2:3], 10)
    c4 = int(myqra[3:4], 10)
    if (len(myqra) > 4):
      c5 = int(myqra[4:5], 36)-10
      c6 = int(myqra[5:6], 36)-10
      myLat = (((c2 * 10) + c4 + ((c6 + 0.5)/24)) - 90)
      myLon = (((c1 * 20) + (c3 * 2) + ((c5 + 0.5) / 12)) - 180)
    else:
      myLat = (((c2 * 10) + c4 ) - 90)
      myLon = (((c1 * 20) + (c3 * 2)) - 180)
    return (myLat, myLon)

def getCookies(myQRA, myOffset):
    c = Cookie.SimpleCookie()
    try:
        c.load(os.environ["HTTP_COOKIE"])
    except KeyError:
        pass
    try:
        if (c["qra"].value != myQRA):
            c["qra"]= myQRA
            c["offset"]= myOffset
            return c
        else:
            return None
    except KeyError:
        c["qra"]= myQRA
        c["offset"]= myOffset
        return c

def readDX(filename):
    return etree.parse(filename).getroot()

def serveUp(myQRA, offset, myLat, myLon):
    print("Content-type: application/xml\n")
    dx = readDX(dxFile)
    dx.xpath("/entries")[0].attrib['qra']=myQRA
    dx.xpath("/entries")[0].attrib['offset']=str(offset)
    addBearings(myLat, myLon, offset, dx)
    print(etree.tostring(dx))

def process():
    myQRA=None
    myLat=None
    myLon=None
    etagIn=None
    etagOut=None
    form = cgi.FormContentDict()
    myOffset=0

    if (form.has_key("offset")):
        myOffset=form.get("offset")[0]

    if (form.has_key("qra")):
        myQRA=form.get("qra")[0]

    if (myQRA != None):
        (myLat,myLon)=qra(myQRA)
        cookieHeader=getCookies(myQRA, myOffset)
        if (cookieHeader != None):
            print(cookieHeader)

    if (myQRA):
        dxFileDate = str(os.stat(dxFile).st_mtime)
        etagKey = myQRA+":"+str(dxFileDate)+":"+str(myOffset)
        etagOut = base64.encodestring(md5.new(etagKey).digest())[:-1]
        print("ETag: ",etagOut)
        try:
            etagIn=os.environ["HTTP_IF_NONE_MATCH"]
        except KeyError:
            pass
        if (etagIn != None and etagIn == etagOut):
            print("Status: 304 Not Modified\n")
        else:
            serveUp(myQRA, myOffset, myLat, myLon)

dxFile="/home/graflex/website/klotz/wa5znu/2007/05/dxqra/dx.xml"
process()
